@@ -101,6 +101,7 @@
 (define (GetTypeAsLilypond)   ; You will likely need (GetContextAsLilypond) also. TODO: Replace with real information, derived from Lilypond
 (define type (string->symbol (d-GetType)))
     (case type  ; Convert Denemo type to lilypond type
+		((LILYDIRECTIVE) (d-DirectiveGet-standalone-grob (d-DirectiveGetForTag-standalone)))
         ((TIMESIG) "TimeSignature") 
         ((CHORD) "NoteHead") ; Rests will be detected as CHORD but it will not work
         ((KEYSIG) "KeySignature")
@@ -407,10 +408,61 @@
 
 (define (DefaultDenemoPlay)
    (d-Play "(display \"Here endeth a scripted playback\")"))
-
 (define (DenemoPlay)
-  (DefaultDenemoPlay))
-
+	(DefaultDenemoPlay))
+;;; DenemoPlay
+(define-once DenemoPlay::Pause #f)
+(define (DenemoPlay::scroll)
+   (define (GetMeasureDuration)
+		(let ((end #f)(start (d-GetTimeAtCursor))(obj (d-GetHorizontalPosition)))	
+		 
+		 (if (d-MoveToMeasureRight)
+				(begin
+					(set! end (d-GetTimeAtCursor))
+					(d-MoveToMeasureLeft)
+					(while (and (not (eq? (d-GetHorizontalPosition) obj)) (d-MoveCursorRight))))
+				(set! end start))
+			(- end start)))
+	(define thetime (inexact->exact (round (* 1000 (GetMeasureDuration)))))
+	(if (and (d-AudioIsPlaying) (not DenemoPlay::Pause))
+		(d-OneShotTimer thetime "(d-MoveToMeasureRight) (d-ScrollRight) (DenemoPlay::scroll)")))
+		
+(define (DenemoPlayScroll)
+	(if (d-AudioIsPlaying)
+		(begin
+			(set! DenemoPlay::Pause #t)
+			(d-Play "(display \"paused scroll play\")"))
+		(begin
+			(d-CreateTimebase)
+			;;move cursor to playback start
+			(let ((on (d-AdjustPlaybackStart 0.0)))
+				(while (and (< (d-GetMidiOnTime) on) (d-MoveToMeasureRight)))
+				(while (and (> (d-GetMidiOnTime) on) (d-MoveToMeasureLeft)))
+				(while (and (< (d-GetMidiOnTime) on) (d-MoveCursorRight))))
+			(d-Play "(disp \"End of scrolled play\")")
+			(set! DenemoPlay::Pause #f)
+			(d-OneShotTimer 50 "(DenemoPlay::scroll)"))))
+			
+;;;;PlayIntro and then cursor to end - assumes playalong set.
+(define (DenemoPlayIntroToEnd)
+	(d-PushPosition)
+	(d-SetPlaybackInterval #t (- (d-GetMidiOnTime) 0.001))
+	;(DenemoSetPlaybackEnd)
+	(d-MoveToMeasureLeft)
+	(while (not (Music?))
+		(d-MoveCursorRight))
+	(DenemoSetPlaybackStart)
+	(d-TogglePlayAlong)
+	(d-Play "(d-PopPosition)(d-TogglePlayAlong)(d-DenemoPlayCursorToEnd)"))
+		
+(define (DenemoPlayAlongPlay) (disp "Reached DenemoPlayAlongPlay now\n")
+	(d-CreateTimebase)
+	(d-SetPlaybackInterval (d-GetMidiOnTime) -1)
+	(define midi (d-GetMidi))
+	(disp "Starting play along play")
+	(d-SetMidiCapture #f)
+	(d-PutMidi midi)
+	(d-Play "(disp \"Play (along) finished\")"))
 ;Scripted start stop - use on Windows.
 (define-once DenemoPaused? #f)
 (define-once DenemoPauseTime 0) ;time to start playing at when un-pausing
@@ -1212,16 +1264,23 @@
 
 ;;;
 (define (DenemoGetDuration title)
-    (TitledRadioBoxMenuList title
+    (define response (TitledRadioBoxMenuList title
         (list (cons "𝅝" "1/1")
         (cons "𝅗𝅥" "1/2")
         (cons "𝅘𝅥" "1/4")
         (cons "𝅘𝅥𝅮" "1/8") 
         (cons "𝅘𝅥𝅯" "1/16")
-        (cons "𝅘𝅥𝅱" "1/32")
-        (cons "𝅘𝅥𝅱" "1/64")
-        (cons "𝅘𝅥𝅱" "1/128"))))
-                            
+        (cons "𝅘𝅥𝅰 " "1/32")
+        (cons "𝅘𝅥𝅱 " "1/64")
+        (cons "𝅘𝅥𝅲 " "1/128")
+        (cons (_ "Custom Duration") #f))))
+   (if response
+		response
+		(let ((response
+					(d-GetUserInput (_ "Horizontal Spacing Basis") (_ "Give fraction of whole note 𝅝   (e.g. 2/1 or 1/7 etc)") "1/5")))
+			(if (and (string? response) (string->number response))
+				response
+				"1/1"))))                   
 ;;
 (define (DenemoSetTitles tag param editing)
     (let ((score (equal? tag "ScoreTitles"))
@@ -1245,60 +1304,50 @@
             (if editing
                 (let ((response  (d-GetUserInputWithSnippets  (if score (_ "Score Titles")  (_ "Movement Titles")) (string-append (_ "Give ") field) initial)))
                     (if response
-                        (car response)
+                        (string-append "\\column{" (cdr response) "}")
                         #f))
                 (let ((response (d-GetUserInput (if score (_ "Score Titles")  (_ "Movement Titles")) (string-append (_ "Give ") field) initial)))
                     (if response (lilypond-markup-escape response) #f))))
             
         (define (write-titles)
-            (let ((header ""))
+            (let ((header " ")(prefix ""))
                 (define (url type)
                     (if score
                             (string-append "\\with-url #'\"scheme:(DenemoSetTitles \\\"" tag "\\\" '" type " #t)\" ")
-                            (string-append "\\with-url #'\"scheme:(d-GoToPosition " (number->string (d-GetMovement)) " 1 1 1)(DenemoSetTitles \\\"" tag "\\\" '" type " #t)\"")))
-            
+                            (string-append "\\with-url #'\"scheme:(d-GoToPosition " (number->string (d-GetMovement)) " 1 1 1)(DenemoSetTitles \\\"MovementTitles\\\" '" type " #t)\"")))
+				(if (not score) (set! prefix " m"))
+				
                 (if dedication
-                                (set! header (string-append header "     dedication = \\markup " (url "dedication") " {" dedication "}\n"))
-                                (set! header (string-append header "     dedication = ##f\n")))
+                                (set! header (string-append header prefix "dedication = \\markup " (url "dedication") " {" dedication "}\n")))
                 (if title
-                                (set! header (string-append header "     title = \\markup " (url "title") " {" title "}\n"))
-                                (set! header (string-append header "     title = ##f\n")))
+                                (set! header (string-append header prefix "title = \\markup " (url "title") " {" title "}\n")))
                 (if subtitle
-                                (set! header (string-append header "     subtitle = \\markup " (url "subtitle") " {" subtitle "}\n"))
-                                (set! header (string-append header "     subtitle = ##f\n")))
+                                (set! header (string-append header prefix "subtitle = \\markup " (url "subtitle") " {" subtitle "}\n")))
                 (if subsubtitle
-                                (set! header (string-append header "     subsubtitle = \\markup " (url "subsubtitle") " {" subsubtitle "}\n"))
-                                (set! header (string-append header "     subsubtitle = ##f\n")))
+                                (set! header (string-append header prefix "subsubtitle = \\markup " (url "subsubtitle") " {" subsubtitle "}\n")))
                 (if instrument
-                                (set! header (string-append header "     instrument = \\markup " (url "instrument") " {" instrument "}\n"))
-                                (set! header (string-append header "     instrument = ##f\n")))
+                                (set! header (string-append header prefix "instrument = \\markup " (url "instrument") " {" instrument "}\n")))
                 (if poet
-                                (set! header (string-append header "     poet = \\markup " (url "poet") " {" poet "}\n"))
-                                (set! header (string-append header "     poet = ##f\n")))
+                                (set! header (string-append header prefix "poet = \\markup " (url "poet") " {" poet "}\n")))
                 (if composer
-                                (set! header (string-append header "     composer = \\markup " (url "composer") " {" composer "}\n"))
-                                (set! header (string-append header "     composer = ##f\n")))
+                                (set! header (string-append header prefix "composer = \\markup " (url "composer") " {" composer "}\n")))
                 (if meter
-                                (set! header (string-append header "     meter = \\markup " (url "meter") " {" meter "}\n"))
-                                (set! header (string-append header "     meter = ##f\n")))
+                                (set! header (string-append header prefix "meter = \\markup " (url "meter") " {" meter "}\n")))
                 (if arranger
-                                (set! header (string-append header "     arranger = \\markup " (url "arranger") " {" arranger "}\n"))
-                                (set! header (string-append header "     arranger = ##f\n")))
+                                (set! header (string-append header prefix "arranger = \\markup " (url "arranger") " {" arranger "}\n")))
                 (if tagline
-                                (set! header (string-append header "     tagline = \\markup " (url "tagline") " {" tagline "}\n"))
-                                (set! header (string-append header "     tagline = ##f\n")))
+                                (set! header (string-append header prefix "tagline = \\markup " (url "tagline") " {" tagline "}\n")))
                 (if copyright
-                                (set! header (string-append header "     copyright = \\markup " (url "copyright") " {" copyright "}\n"))
-                                (set! header (string-append header "     copyright = ##f\n")))
+                                (set! header (string-append header prefix "copyright = \\markup " (url "copyright") " {" copyright "}\n")))
                 (if piece
-                                (set! header (string-append header "     piece = \\markup " (url "piece") " {" piece "}\n"))
-                                (set! header (string-append header "     piece = ##f\n")))
+                                (set! header (string-append header prefix "piece = \\markup " (url "piece") " {" piece "}\n")))
                 (if opus
-                                (set! header (string-append header "     opus = \\markup " (url "opus") " {" opus "}\n"))
-                                (set! header (string-append header "     opus = ##f\n")))
+                                (set! header (string-append header prefix "opus = \\markup " (url "opus") " {" opus "}\n")))
+                          
                 (d-SetSaved #f)
                 
-                (DenemoPrintAllHeaders);;; here to disable book titles.
+                (if (not (eq? param 'initialize)) 
+					(DenemoPrintAllHeaders));;; here to disable book titles.
                 (if (not (d-Directive-header? "MovementTitles"))
                  (d-DirectiveDelete-paper "PrintAllHeaders"))
              
@@ -1312,35 +1361,114 @@
                         (d-DirectivePut-header-display tag (_ "Movement Titles"))
                         (d-DirectivePut-header-override tag DENEMO_OVERRIDE_GRAPHIC)))
                 ;;; if setting movement titles but no score titles are set then initialize score titles to #f
-                (if (and (not score) (not (d-Directive-scoreheader? "ScoreTitles")))
-                        (DenemoSetTitles "ScoreTitles" 'initialize #f))))
+                ;(if (and (not score) (not (d-Directive-scoreheader? "ScoreTitles")))
+                ;        (DenemoSetTitles "ScoreTitles" 'initialize #f))
+                        ))
                         
     (define (form-pair name title)
         (string-append "(cons '" name (if (string? title) (string-append " \"" (scheme-escape title) "\"") " #f") ")"))
+        
+    (define (set-data)
+		(set! data (eval-string data))
+		(set! dedication (assq-ref data 'dedication))
+		(set! title (assq-ref data 'title))
+		(set! subtitle (assq-ref data 'subtitle))
+		(set! subsubtitle (assq-ref data 'subsubtitle))
+		(set! instrument (assq-ref data 'instrument))
+		(set! poet (assq-ref data 'poet))
+		(set! composer (assq-ref data 'composer))
+		(set! meter (assq-ref data 'meter))
+		(set! arranger (assq-ref data 'arranger))
+		(set! tagline (assq-ref data 'tagline))
+		(set! copyright (assq-ref data 'copyright))
+		(set! piece (assq-ref data 'piece))
+		(set! opus (assq-ref data 'opus)))
 ;;; procedure starts here
+
+
+		(if (not score)
+			(begin (disp "starting"          (d-Directive-score? "MovementTitles") "\n\n"            )
+				(if (not (d-Directive-score? "MovementTitles"))
+					(begin
+(d-DirectivePut-score-override "MovementTitles" DENEMO_OVERRIDE_AFFIX)				
+(d-DirectivePut-score-prefix "MovementTitles" "
+\\paper {
+scoreTitleMarkup = \\markup { \\column {
+\\override #'(baseline-skip . 3.5)
+\\column {
+	\\fill-line { \" \"}
+	\\fill-line { \\fromproperty #'header:mdedication }
+	\\override #'(baseline-skip . 3.5)
+	\\column {
+	  \\fill-line {
+		\\huge \\larger \\bold
+		\\fromproperty #'header:mtitle
+	  }
+	  \\fill-line {
+		\\bold
+		\\fromproperty #'header:msubtitle
+	  }
+	  \\fill-line {
+		\\smaller \\bold
+		\\fromproperty #'header:msubsubtitle
+	  }
+	  \\fill-line {
+		\\fromproperty #'header:mpoet
+		{ \\large \\bold \\fromproperty #'header:minstrument }
+		\\fromproperty #'header:mcomposer
+	  }
+	  \\fill-line {
+		\\fromproperty #'header:mmeter
+		\\fromproperty #'header:marranger
+		}
+	\\fill-line {
+		\\fromproperty #'header:mpiece
+		\\fromproperty #'header:mopus
+	  }
+  \\fill-line {
+	\\fromproperty #'header:mverses
+	  }
+	}
+  }
+}
+}
+}")))
+		(set! tag "MTitles")
+		(set! data (d-DirectiveGet-header-data "MovementTitles"))
+		(if data ;legacy movement titles
+			(begin
+				(set-data)
+				(let ((thealist (string-append "(list " 
+						(form-pair "dedication" dedication) 
+						(form-pair "title" title)
+						(form-pair "subtitle" subtitle)
+						(form-pair "subsubtitle" subsubtitle)
+						(form-pair "instrument" instrument)
+						(form-pair "poet" poet)
+						(form-pair "composer" composer)
+						(form-pair "meter" meter)
+						(form-pair "arranger" arranger)
+						(form-pair "tagline" tagline)
+						(form-pair "copyright" copyright)
+						(form-pair "piece" piece)
+						(form-pair "opus" opus) " '())")))
+				 (d-DirectivePut-header-data tag thealist) ;(disp "we have thealist new format " tag " " thealist "\n\n")
+				 (d-DirectiveDelete-header "MovementTitles")
+				 ))))) ;end of if movement titles, check for legacy ones and create the procedure scoreTitleMarkup for writing them
+				
+
+
         (if score
             (set! data (d-DirectiveGet-scoreheader-data tag))
             (set! data (d-DirectiveGet-header-data tag)))
+        ;(disp "so data we get for " tag  " is " data "\n\n")
         (if data 
-            (begin
-                (set! data (eval-string data))
-                (set! dedication (assq-ref data 'dedication))
-                (set! title (assq-ref data 'title))
-                (set! subtitle (assq-ref data 'subtitle))
-                (set! subsubtitle (assq-ref data 'subsubtitle))
-                (set! instrument (assq-ref data 'instrument))
-                (set! poet (assq-ref data 'poet))
-                (set! composer (assq-ref data 'composer))
-                (set! meter (assq-ref data 'meter))
-                (set! arranger (assq-ref data 'arranger))
-                (set! tagline (assq-ref data 'tagline))
-                (set! copyright (assq-ref data 'copyright))
-                (set! piece (assq-ref data 'piece))
-                (set! opus (assq-ref data 'opus))))
+            (set-data))
         (if (not data)
             (set! data '()))
 
-        (let ((choice (list 
+        (let ((choice (list         
+                (cons (_ "FINISH") 'abort)
                 (cons (_ "dedication") 'dedication)
                 (cons (_ "title") 'title)
                 (cons (_ "subtitle") 'subtitle)
@@ -1458,7 +1586,8 @@
                     (if opus (set! editing #t))                
                     (set! choice (get-field (_ "opus") opus))
                     (if choice (set! opus choice))))
-                    
+             (if (not choice) 
+				(set! choice 'abort))       
              
              (if (not (eq? choice 'abort))
                 (let ((thealist (string-append "(list " 
@@ -1478,7 +1607,9 @@
                     (if score
                         (d-DirectivePut-scoreheader-data tag thealist)
                         (d-DirectivePut-header-data tag thealist))
-                    (write-titles))))))
+                    (write-titles)
+                    (if (not param)
+						(DenemoSetTitles tag param editing)))))))
                     
 ;;;;;;;;;;;
 (define (DenemoSetVerticalSpacingDist tag type title default)
@@ -1886,3 +2017,30 @@
 	
 (define (ToggleViewVisibility name)
 	(d-SetViewVisibility name (not (d-GetViewVisibility name))))	
+
+;;;;;;;;;VerboseLilyPondView - directives in the scoreblock are annotated within the LilyPond syntax to describe their origin 
+(define (AnnotateDirectives field)
+	(define nth (eval-string (string-append "d-DirectiveGetNthTag-" field)))
+	(let loop ((count 0))
+		(define tag (nth count))
+		(if tag 
+			(let ((prefix (eval-string (string-append "(d-DirectiveGet-" field "-prefix \"" tag "\")")))
+				  (postfix (eval-string (string-append "(d-DirectiveGet-" field "-postfix \"" tag "\")")))
+				  (show (d-GetLabel tag)))
+				  (if (not show) (set! show "")) 
+				  (if (and prefix (not (equal? (string-ref prefix 0) #\%)))
+						(eval-string (string-append "(d-DirectivePut-" field "-prefix  \"" tag "\" \"%{" show ":(" tag ")%} " (scheme-escape prefix) "\")")))
+				  (if (and postfix (not (equal? (string-ref postfix 0) #\%)))
+						(eval-string (string-append "(d-DirectivePut-" field "-postfix  \"" tag "\" \"%{" show ":(" tag ")%} " (scheme-escape postfix) "\")")))
+				  (loop (1+ count))))))
+(define (AnnotateScoreDirectives)			
+		(AnnotateDirectives "score")
+		(AnnotateDirectives "scoreheader")
+		(AnnotateDirectives "paper")
+		(AnnotateDirectives "header")
+		(AnnotateDirectives "layout")
+		(ForEachStaffInScore 
+			"(AnnotateDirectives \"movementcontrol\")
+			(AnnotateDirectives \"staff\")
+			(AnnotateDirectives \"voice\")"))
+;;;;;;;;;;;;;;;;;;
