@@ -15,7 +15,7 @@
 #define AUDIOINTERFACE_H
 
 #include <denemo/denemo_types.h>
-
+#include "smf.h"
 
 /**
  * The common interface for all audio and MIDI backends.
@@ -209,7 +209,7 @@ int panic (backend_type_t backend);
  * Sends a MIDI panic CC and/or resets the synth engine for all active
  * backends.
  */
-int panic_all ();
+int panic_all (void);
 
 
 
@@ -271,7 +271,7 @@ void queue_redraw_all ();
 /**
  * Queues a redraw of the playhead.
  */
-void queue_redraw_playhead ();
+void queue_redraw_playhead (smf_event_t * event);
 
 extern GMutex smfmutex;
 
