@@ -788,12 +788,18 @@ initialize_until_time (void)
     play_until = G_MAXDOUBLE;
 }
 
-//test if the midi event in buf is a note-on for the current note
+//for PLAYALONG:
+//Play the passed midi event  in buf. 
+// Test if the midi event is a note-on for the current note
 //if so set play_until
 //advance cursor to next note
 static void
 advance_until_time (gchar * buf)
 {
+  gboolean playnote = TRUE; //don't play correct notes
+  gboolean conduct = Denemo.project->midi_destination & MIDICONDUCT;
+  highlight_playback_note(0xb5);
+  //g_print ("RESET\n");
   if (Denemo.project->movement->currentobject)
     {
       DenemoObject *obj = Denemo.project->movement->currentobject->data;
@@ -807,9 +813,11 @@ advance_until_time (gchar * buf)
           if (thechord->notes)
             {
               note *thenote = thechord->notes->data;
-              if (((buf[0] & 0xf0) == MIDI_NOTE_ON) && buf[2] && buf[1] == (dia_to_midinote (thenote->mid_c_offset) + thenote->enshift))
+              if (((buf[0] & 0xf0) == MIDI_NOTE_ON) && buf[2] && 
+				(conduct || (buf[1] == (dia_to_midinote (thenote->mid_c_offset) + thenote->enshift))))
                 {
                   gdouble thetime = get_time ();
+                  highlight_playback_note(0x0);
                   Denemo.project->movement->start_player = thetime - obj->earliest_time;
 
                   if ((!Denemo.prefs.ignore_ties) && thechord->is_tied && cursor_to_next_note ())
@@ -831,6 +839,7 @@ advance_until_time (gchar * buf)
                           play_until = obj->earliest_time - SHAVING;
                           //g_debug("play until %f\n", play_until);
                         }
+                       playnote = FALSE; //do not play note as it will be played anyway
                     }
                   while (!thechord->notes);
                 }
@@ -841,6 +850,8 @@ advance_until_time (gchar * buf)
     }
   else
     g_warning ("Not on an object");
+ if ((!conduct) && playnote)
+	play_adjusted_midi_event (buf); //play the note, even if it is wrong
 }
 
 static void
@@ -880,6 +891,8 @@ play_adjusted_midi_event (gchar * buf)
 void
 handle_midi_event (gchar * buf)
 {
+  if (Denemo.transpose_midi_in && ((((*buf)&0xF0) == MIDI_NOTE_ON) || (((*buf)&0xF0) == MIDI_NOTE_OFF)))
+	*(buf+1) +=  Denemo.transpose_midi_in;
   //g_print ("play adj midibytes 0x%hhX 0x%hhX 0x%hhX\n", *(buf+0), *(buf+1), *(buf+2));
   if (Denemo.prefs.notesonlymidiin && (((*buf)&0xF0) != MIDI_NOTE_ON) && (((*buf)&0xF0) != MIDI_NOTE_OFF))
         {//g_print ("Dropping messages other than note on/off\n"); 
