@@ -17,28 +17,29 @@
 #if GTK_MAJOR_VERSION==3
 #include <gdk/gdkkeysyms-compat.h>      //FIXME Look for something more gtk3 like
 #endif
-static GdkEventKey **divert_key_event;  /* Non null if key events are being intercepted by a function
-                                         * (which is running a gtk_mail_loop() for this reason).
-                                         * return TRUE if a key press successfully captured
-                                         * in which case the kyval, state pair are returned
-                                         */
+
+
 
 static gint divert_key_id = 0;
+/* Non null if key events are being intercepted by function 
+										 * intercept_scorearea_keypress (GdkEventKey * pevent)
+                                         * (which runs a gtk_mail_loop() which is exited when a keypress arrives).
+                                         * return TRUE when a key press successfully captured
+                                         * and the passed in event pointer is set to a copy of the event
+                                         */
 gboolean
 intercept_scorearea_keypress (GdkEventKey * pevent)
 {
-  if (divert_key_event)
+  if (Denemo.project->movement->divert_key_event)
     {
-      warningdialog (_("Recursive key capture not possible!"));    /* we could make a stack of them instead ... */
+      warningdialog (_("Recursive key capture not possible!\nBring the Denemo Display into focus and press a key there to clear this condition."));    /* we could make a stack of them instead ... */
       return FALSE;
     }
   GdkEventKey *event;
   divert_key_id = Denemo.project->id;
-  divert_key_event = &event;
+  Denemo.project->movement->divert_key_event = &event;
   gtk_main ();
-  divert_key_event = NULL;
-  // *keyval = event->keyval;
-  // *state = dnm_sanitize_key_state(event);
+  Denemo.project->movement->divert_key_event = NULL;
   memcpy (pevent, event, sizeof (GdkEventKey));
   return TRUE;
 }
@@ -195,6 +196,8 @@ perform_command (const gchar * command_name, GdkEventKey * event)
 #endif
   gchar *trunc = g_strdup (lookup_label_from_idx (Denemo.map, Denemo.LastCommandId));  
   truncate_label (NULL, trunc);
+  gchar *c = trunc;
+  for(;*c;c++) if (*c=='&') *c='+';//ampsersand not allowed in markup FIXME replace with &amp; fix code duplication with menusystem.c
   g_string_printf (Denemo.input_filters, "%s  <span foreground=\"Green\">%s.</span> <span foreground=\"blue\">%s</span>", trunc, _("Help"), _("(use Fn12 to Repeat)"));
   write_input_status();
   g_free (trunc);
@@ -203,7 +206,7 @@ perform_command (const gchar * command_name, GdkEventKey * event)
 
 //return the value of perform_command if executed or "" if keypress is part of a two-key shortcut, or NULL toherwise
 gchar *
-process_key_event (GdkEventKey * event, gchar * perform_command ())
+process_key_event (GdkEventKey * event, gchar * perform_command (const gchar * command_name, GdkEventKey * event))
 {
   keymap *the_keymap = Denemo.map;
   //g_debug("\n********\nCaps Lock %x?\n\n********\nShifted %x?\n", event->state&GDK_LOCK_MASK,          event->state&GDK_SHIFT_MASK     );
@@ -254,7 +257,9 @@ process_key_event (GdkEventKey * event, gchar * perform_command ())
           return NULL;
         }
     }
-
+    if (event->keyval == GDK_Return)
+		call_out_to_guile ("(d-HighlightCursor)");
+		
   /*  we create a store for the prefix char and look to see if it is populated when a keystroke is received. If it is populated, we try for the two-key combination, {???else we try for the single key, and if that fails populate the store. OR if it fails clear store}. If the two-key combination works we clear the store. If the two-key combination fails we try for the single key, if that succeeds we clear the store if it fails we set the store to the unresolved keystroke.  */
 
   gchar *ret = NULL;
@@ -354,7 +359,7 @@ process_key_event (GdkEventKey * event, gchar * perform_command ())
 gint
 scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
 {
-  //g_print ("Scorearea key press event: keyval %d (%s), string |%s|, length %d, state %x, keycode %d, group %d, is_modifier flag %d\n", event->keyval, gdk_keyval_name(event->keyval), event->string, event->length, event->state, event->hardware_keycode, event->group, event->is_modifier);
+  //g_print ("Scorearea key press event: keyval %d (%s), string |\n%s\n|, length %d, state %x, keycode %d, group %d, is_modifier flag %d\n", event->keyval, gdk_keyval_name(event->keyval), event->string, event->length, event->state, event->hardware_keycode, event->group, event->is_modifier);
   if ((event->keyval == 65481) && (event->state == 0)) //Fn12 hardwired to repeat last command
     {
       if (Denemo.LastCommandId != -1)
@@ -398,10 +403,10 @@ scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
   //g_print("press Denemo %x state %x klock %x\n", Denemo.keyboard_state, event->state, klock_mask(event->keyval));
 
   //g_debug("State eored %x\n", (lock_mask(event->keyval)^event->state));
-  if (divert_key_event && !isModifier (event) && divert_key_id == Denemo.project->id)
+  if (Denemo.project->movement->divert_key_event && !isModifier (event) && divert_key_id == Denemo.project->id)
     {
       dnm_sanitize_key_state (event);
-      *divert_key_event = event;
+      *Denemo.project->movement->divert_key_event = event;
       //g_object_ref(event); FIXME do we need to keep it around?
       gtk_main_quit ();
       return TRUE;              //*is* reached main loop exits to the caller of the loop when it next gains control
