@@ -240,6 +240,11 @@ determineDuration (gint duration, gchar ** durationName)
         {
           return determineDuration (-duration, durationName);
         }
+     else  if (duration > 7)
+        {
+           *durationName = "whole";
+			g_critical ("chord with baseduration set to unusual value %d", duration);
+        }        
       break;
     }
 }
@@ -344,7 +349,7 @@ newSourceFileElem (xmlNodePtr curElem, xmlNsPtr ns, DenemoProject * gui)
 #ifndef USE_EVINCE
   g_debug ("This feature requires denemo to be built with evince");
 #else
-  if (source_position (&gui->source_x, &gui->source_y, &gui->source_width, &gui->source_height, &gui->source_scale))
+  if (Denemo.project->source_scale)
     {
       xmlNodePtr sourceElem = xmlNewChild (curElem, ns, (xmlChar *) "sourcefile", NULL);
       newXMLIntChild (sourceElem, ns, (xmlChar *) "x", gui->source_x);
@@ -614,8 +619,15 @@ newVoiceProps (xmlNodePtr parentElem, xmlNsPtr ns, DenemoStaff * curStaffStruct)
 {
   xmlNodePtr curElem = xmlNewChild (parentElem, ns, (xmlChar *) "voice-props", NULL);
   newXMLIntChild (curElem, ns, (xmlChar *) "number-of-lines", curStaffStruct->no_of_lines);
+  if (curStaffStruct->fixed_height)
+	{
+	  newXMLIntChild (curElem, ns, (xmlChar *) "space_above", curStaffStruct->space_above);		
+	  newXMLIntChild (curElem, ns, (xmlChar *) "space_below", curStaffStruct->space_below);		
+	}
   if (curStaffStruct->color)
     newXMLIntChild (curElem, ns, (xmlChar *) "staff-color", curStaffStruct->color);
+  if (curStaffStruct->type)
+    xmlNewChild (curElem, ns, (xmlChar *) "staff-type", curStaffStruct->type);
   if (curStaffStruct->hidden)
     newXMLIntChild (curElem, ns, (xmlChar *) "staff-hidden", curStaffStruct->hidden);
   if (curStaffStruct->range)
@@ -810,7 +822,7 @@ parseObjects (xmlNodePtr measureElem, xmlNsPtr ns, GList * curObjNode)
                 newDirectivesElem (objElem, ns, (thechord)->directives, "directives");
               }
             if (thechord->baseduration < -7) //breve etc have negative of actual ticks stored here
-              newXMLIntChild (objElem, ns, (xmlChar *) "ticks", -thechord->baseduration);
+              newXMLIntChild (objElem, ns, (xmlChar *) "ticks", curObj->durinticks);// -thechord->baseduration);
             else if (thechord->baseduration < 0) //a swung note, swung notes have negative of the nominal duration, usually 1/8th notes ie 3
               newXMLIntChild (objElem, ns, (xmlChar *) "ticks", curObj->durinticks);
             /*
@@ -1169,8 +1181,8 @@ exportXML (gchar * thefilename, DenemoProject * gui)
   GList *conditions;
   for (conditions = gui->criteria; conditions; conditions = conditions->next)
     {
-      DenemoOmissionCriterion *condition = (DenemoOmissionCriterion *)conditions->data;
-      xmlNewTextChild (scoreElem, ns, (xmlChar *) "omission-criterion", (xmlChar *) (condition->name));
+      DenemoInclusionCriterion *condition = (DenemoInclusionCriterion *)conditions->data;
+      xmlNewTextChild (scoreElem, ns, (xmlChar *) "Inclusion-criterion", (xmlChar *) (condition->name));
     }
     
   /* lilycontrol for the whole musical score */
@@ -1233,6 +1245,9 @@ exportXML (gchar * thefilename, DenemoProject * gui)
         newXMLIntChild (parentElem, ns, (xmlChar *) "page-width", si->page_width);
       if (si->page_height)
         newXMLIntChild (parentElem, ns, (xmlChar *) "page-height", si->page_height);
+      if (si->staffspace != DENEMO_INITIAL_STAFF_HEIGHT)
+        newXMLIntChild (parentElem, ns, (xmlChar *) "staffspace", si->staffspace);
+ 
       if (si->measurewidth != DENEMO_INITIAL_MEASURE_WIDTH)
         newXMLIntChild (parentElem, ns, (xmlChar *) "measure-width", si->measurewidth);
 
@@ -1259,13 +1274,13 @@ exportXML (gchar * thefilename, DenemoProject * gui)
         newScrollPointsElem (mvmntElem, ns, si->scroll_points);
       
       // output audio source
-      if (si->recording)
+      if (si->recording && (si->recording->type==DENEMO_RECORDING_AUDIO))
         outputAudio (mvmntElem, ns, si->recording);
 
       parentElem = xmlNewChild (mvmntElem, ns, (xmlChar *) "score-info", NULL);
       curElem = xmlNewChild (parentElem, ns, (xmlChar *) "tempo", NULL);
-      newXMLFraction (xmlNewChild (curElem, ns, (xmlChar *) "duration", NULL), ns, 1, 4);
-      newXMLIntChild (curElem, ns, (xmlChar *) "bpm", si->tempo * si->master_tempo);
+      //newXMLFraction (xmlNewChild (curElem, ns, (xmlChar *) "duration", NULL), ns, 1, 4); do not create this, it is always a quarter note
+      newXMLIntChild (curElem, ns, (xmlChar *) "bpm", si->tempo);
 
       stavesElem = xmlNewChild (mvmntElem, ns, (xmlChar *) "staves", NULL);
       for (curStaff = si->thescore; curStaff != NULL; curStaff = curStaff->next)
@@ -1363,13 +1378,21 @@ exportXML (gchar * thefilename, DenemoProject * gui)
         }                       /* end for each voice in score */
     }                           // for each movement
   /* Save the file. */
-
-  xmlSaveCtxt *ctxt = xmlSaveToFilename (filename->str, "UTF-8", XML_SAVE_FORMAT | XML_SAVE_NO_EMPTY);
-  if (!ctxt || xmlSaveDoc (ctxt, doc) < 0 || xmlSaveClose (ctxt) < 0)
-    {
-      g_warning ("Could not save file %s", filename->str);
-      ret = -1;
-    }
+  if (g_strrstr (filename->str, "examples") || g_strrstr (filename->str, "blank.denemo") || g_strrstr (filename->str, "tests/tmp") ) //examples directory is used for tests, save uncompressed
+	{
+		xmlSaveCtxt *ctxt = xmlSaveToFilename (filename->str, "UTF-8", XML_SAVE_FORMAT | XML_SAVE_NO_EMPTY);
+		if (!ctxt || xmlSaveDoc (ctxt, doc) < 0 || xmlSaveClose (ctxt) < 0)
+		{
+		  g_warning ("Could not save file %s", filename->str);
+		  ret = -1;
+		}	
+	}
+  else 
+	  if (xmlSaveFormatFile (filename->str, doc, 1) < 0)
+		{
+		  g_warning ("Could not save file %s", filename->str);
+		  ret = -1;
+		}
 
   /* Clean up all the memory we've allocated. */
 
