@@ -155,7 +155,7 @@ process_command_line (int argc, char **argv, gboolean gtkstatus)
                         _("Denemo is a graphical music notation editor.\n"
                           "It uses GNU Lilypond for music typesetting.\n"
                           "Denemo is part of the GNU project."), NULL);
-  const gchar* footer = _("Report bugs to http://www.denemo.org\n"
+  const gchar* footer = _("Report bugs to https://www.denemo.org\n"
                           "GNU Denemo, a free and open music notation editor");
   Denemo.LastCommandId = -1;
   context = g_option_context_new (subtitle);
@@ -210,25 +210,21 @@ init_environment()
 #ifdef G_OS_WIN32
   gchar *prefix = g_win32_get_package_installation_directory (NULL, NULL);
   gchar *guile = g_build_filename (prefix, "share", "guile", NULL);
-  gchar *guile_1_8 = g_build_filename (guile, "1.8", NULL);
+  gchar *guile_3_0 = g_build_filename (guile, "3.0", NULL);
   gchar *lilypond_current_scm = g_build_filename (prefix, "share", "lilypond", "current", "scm", NULL);
   gchar *denemo_scm = g_build_filename (get_system_data_dir (), COMMANDS_DIR, NULL);
   gchar *denemo_modules_scm = g_build_filename (get_system_data_dir (), COMMANDS_DIR, "denemo-modules", NULL);
   if (g_file_test (guile, G_FILE_TEST_EXISTS))
     {
-      gchar *guile_path = g_strconcat (guile, ";", guile_1_8, ";", denemo_scm, ";", denemo_modules_scm, ";", lilypond_current_scm, NULL);
+      gchar *guile_path = g_strconcat (guile, ";", guile_3_0, ";", denemo_scm, ";", denemo_modules_scm, ";", lilypond_current_scm, NULL);
       //FIXME TRUE means we overwrite any installed version of lilyponds scm, FALSE risks not putting denemos scm in the path...
       g_setenv ("GUILE_LOAD_PATH", guile_path, TRUE);
       g_info ("Setting GUILE_LOAD_PATH=%s\n", guile_path);
     }
   else
     warningdialog (_("You may need to set GUILE_LOAD_PATH to the directory where you have ice9 installed\n"));
-  g_setenv ("PANGO_PREFIX", prefix, TRUE);
-  g_setenv ("PANGO_MODULE_VERSION", "1.6.0", TRUE);
-  g_setenv ("PANGO_SO_EXTENSION", ".dll", TRUE);
-  g_info ("Setting PANGO_PREFIX=%s\n", prefix);
 
-  g_setenv ("GTK_MODULE_VERSION", "2.10.0", TRUE);
+  g_setenv ("GTK_MODULE_VERSION", "3.0.0", TRUE);
   g_setenv ("GTK_SO_EXTENSION", ".dll", TRUE);
   g_setenv ("GTK_PREFIX", prefix, TRUE);
   g_info ("Setting GTK_PREFIX=%s\n", prefix);
@@ -249,7 +245,7 @@ init_environment()
 
   g_setenv ("PATH", path, TRUE);
   g_info ("PATH set to %s\n", path);
-  gchar *lilypond_data_path = g_build_filename (prefix, "share", "lilypond", "current", NULL);
+  gchar *lilypond_data_path = g_build_filename (prefix, "lilypond", "share", "lilypond", "2.24.4", NULL);
   g_setenv ("LILYPOND_DATA_PATH", lilypond_data_path, FALSE);
   g_info ("LILYPOND_DATA_PATH will be %s if not already set", lilypond_data_path);
 
@@ -438,6 +434,10 @@ main (int argc, char *argv[])
   if(!(gtk_status = gtk_init_check (&argc, &argv)))
     g_message(_("Could not start graphical interface."));
 
+  /* Ensure the correct application icon is displayed with GTK3 under Wayland.
+   * See https://honk.sigxcpu.org/con/GTK__and_the_application_id.html */
+  g_set_prgname ("org.denemo.Denemo");
+
   files = process_command_line (argc, argv, gtk_status);
 
   /* initialization of directory relocatability */
