@@ -244,6 +244,13 @@ put_initialization_script (GtkWidget * widget, gchar * directory)
     }
 }
 
+static void find_directives_created (GtkWidget * widget, gchar *name)
+{
+  gchar *thescript = g_strdup_printf ("(DirectivesFromCommand \"%s\")", name);
+  call_out_to_guile (thescript);
+  g_free (thescript);
+}
+
 
 /* replace dangerous characters in command names */
 static void
@@ -307,7 +314,7 @@ insertScript (GtkWidget * widget, gchar * insertion_point)
       //g_file_set_contents(xml_path, text, -1, NULL);
       save_command_metadata (xml_path, myname, mylabel, mytooltip, idx < 0 ? NULL : after);
       save_command_data (scm_path, myscheme);
-      load_xml_keymap (xml_path);//g_print ("Loading from %s\n", xml_path);
+      load_commands_from_xml (xml_path);//g_print ("Loading from %s\n", xml_path);
 
       if (confirm (_("New Command Added"), _("Do you want to save this with your default commands?")))
         save_accels ();
@@ -489,11 +496,7 @@ toggle_playback_view (DenemoAction * action, gpointer param)
     gtk_widget_hide (w);
   else
     {
-      gtk_widget_show (w);
-      GtkImageType type = gtk_image_get_storage_type (GTK_IMAGE (Denemo.playbackview));
-      if (type == GTK_IMAGE_EMPTY)
-        call_out_to_guile ("(d-PlaybackView #f)");
-
+	 gtk_widget_show (w);
     }
 write_status (Denemo.project);
 }
@@ -647,7 +650,7 @@ write_status (Denemo.project);
 //toggle_rhythm_mode
 
 /**
- *  Function to toggle visibility of print preview pane of current project
+ *  Function to toggle visibility of score area (display) pane of current project
  *
  *
  */
@@ -768,7 +771,10 @@ load_command_from_location (GtkWidget * w, gchar * filepath)
 {
   gchar *location = g_strdup_printf ("%s%c", filepath, G_DIR_SEPARATOR);
   g_info ("Calling the file loader with %s", location);
-  load_keymap_dialog_location (location);
+  if (Denemo.command_manager)
+	warningdialog (_("Unable to load new commands as the Command Center is already created - re-start Denemo to do this"));
+  else
+	load_keymap_dialog_location (location);
   g_free (location);
 }
 
@@ -803,6 +809,8 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
         }
       gchar *trunc = g_strdup (lookup_label_from_idx (Denemo.map, Denemo.LastCommandId));  
       truncate_label (NULL, trunc);
+      gchar *c = trunc;
+      for(;*c;c++) if (*c=='&') *c='+';//ampsersand not allowed in markup
       g_string_printf (Denemo.input_filters, "%s  <span foreground=\"Green\">%s.</span> <span foreground=\"blue\">%s</span>", trunc, _("Help"), _("(use Fn12 to Repeat)"));
       write_input_status();
       g_free (trunc);
@@ -845,7 +853,10 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
       g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (save_default_keymap_file), action);
 
-
+	  item = gtk_menu_item_new_with_label (_("Load Command Set"));
+	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	  g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (load_keymap_dialog), action);
+		
       item = gtk_separator_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
     }                           //idx!=-1
@@ -856,7 +867,7 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
   if (row && !myposition)    //menu item runs a script
     myposition = row->menupath;
 
- //g_print("position is %s\n", myposition);
+ //g_print("position is %s\n", myposition); e.g /ObjectMenu/NotesRests
   if (myposition == NULL)
     {
       g_warning("Cannot find the position of this menu item %s in the menu system", func_name);
@@ -874,6 +885,21 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
       g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (load_command_from_location), (gpointer) filepath);
     }
 
+
+  static gchar *filepath2;
+  if (filepath2)
+	g_free (filepath2);
+    filepath2 = g_build_filename (get_user_data_dir (TRUE), COMMANDS_DIR, "menus", myposition, NULL); //g_print ("%s", filepath2);
+	 if (0 == g_access (filepath2, 4))
+		{
+		  //g_print("We can look for a menu item in the path %s at %p\n", filepath2, filepath2);
+		  item = gtk_menu_item_new_with_label ("My Commands");
+		  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		  g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (load_command_from_location), (gpointer) filepath2);
+		}
+	
+
+
   if (!is_action_name_builtin ((gchar *) func_name))
     {
       gchar *scheme = get_scheme_from_idx (idx);
@@ -884,6 +910,12 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
           item = gtk_menu_item_new_with_label (_("Get Script into Scheme Window"));
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (appendSchemeText_cb), scheme);
+          
+          item = gtk_menu_item_new_with_label (_("Find Directives Created by this Command"));
+		 // gtk_widget_set_sensitive (item, sensitive);
+		  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		  g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (find_directives_created), (gpointer)func_name);
+  
         }
       {
 
@@ -911,7 +943,9 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
       g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (uploadMenuItem), action);
 #endif
     }
+    
 
+  
   item = gtk_menu_item_new_with_label (_("Save Script as New Menu Item"));
   gtk_widget_set_sensitive (item, sensitive);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -934,6 +968,10 @@ menu_click (GtkWidget * widget, GdkEventButton * event, DenemoAction * action)
 
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (put_initialization_script), myposition);
+  
+
+   
+  
 
   gtk_widget_show_all (menu);
   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time ());
@@ -1182,7 +1220,7 @@ static void create_palette_for_menu (GtkWidget *menu)
         if (palette_is_empty)
           {
             delete_palette (pal);
-            warningdialog (_("This menu holds no commands"));
+            warningdialog (_("All the commands in this menu are hidden - use the Command Center to un-hide them or More Commands from the Main Menu"));
           }
         }
     else
@@ -1312,6 +1350,12 @@ static void relabel_tear_off (GtkWidget *menu, gint length)
 */
 void denemo_menusystem_add_command (gchar *path, gchar *name, gchar *after)
 {
+	if (get_menupath_for_name (name)) 
+		{
+			if (!Denemo.old_user_data_dir)
+				g_warning("Not upgrading but already have %s\n", name);
+			return;
+		}
     DenemoAction *action = denemo_menusystem_get_action (name);
     GtkWidget *item, *parent = denemo_menusystem_get_widget (path);
     if (!parent)
@@ -1475,21 +1519,21 @@ void denemo_action_group_add_radio_actions (void)
                     
 DenemoAction *denemo_menusystem_get_action (gchar *name)  
 {
-  return  name?(DenemoAction*)g_hash_table_lookup (Actions, name):NULL;
+  return  (name && Actions)?(DenemoAction*)g_hash_table_lookup (Actions, name):NULL;
  
 }   
 
 GList* denemo_action_get_proxies (DenemoAction *action)
 {
     
-    return action?(GList*)g_hash_table_lookup (ActionWidgets, action->name):NULL;
+    return (action && ActionWidgets)?(GList*)g_hash_table_lookup (ActionWidgets, action->name):NULL;
 }
 
 gchar *denemo_action_get_name (DenemoAction *action) {
-    return action->name;
+    return action?action->name:NULL;
 }
 gchar *denemo_action_get_tooltip (DenemoAction *action) {
-    return action->tooltip;
+    return action?action->tooltip:NULL;
 }
 
 
@@ -1498,7 +1542,7 @@ void denemo_action_activate(DenemoAction *action)
 {
     if(action) 
         {
-        if (action->type)
+        if (action->type && action->callback)
             ((BuiltInCallback)(action->callback))(&DummyAction, NULL);
         else 
             activate_script (action, NULL);
