@@ -29,7 +29,7 @@ static gint readxmlprefsFile (gchar * filename);
 static void set_default_lilypond_path (void)
 {
 #ifdef G_OS_WIN32
-  ret->lilypath = g_string_new (g_build_filename (get_system_bin_dir (), "lilypond-windows.exe", NULL));       //We don't assume the file assoc works - we are installing this anyway to a known place,the option  neither lilypond-windows.exe nor the -dgui option are used
+  ret->lilypath = g_string_new (g_build_filename (get_system_bin_dir (), "..", "lilypond", "bin", "lilypond-windows.exe", NULL));       //We don't assume the file assoc works - we are installing this anyway to a known place,the option  neither lilypond-windows.exe nor the -dgui option are used. There is no more lilypond-windows in 2.24.4. I created a wrapper lilypond-windows.exe to hide the console. 
 #else /* !G_OS_WIN32 */
  #ifdef _GUB_BUILD_
    #ifdef _MACH_O_
@@ -54,11 +54,10 @@ initprefs (void)
   gchar *dotdenemo = (gchar *) get_user_data_dir (TRUE);
   gchar *localrc = dotdenemo ? g_build_filename (dotdenemo, PREFS_FILE, NULL) : NULL;
     if(Denemo.old_user_data_dir) {
-        if(confirm (_("Denemo Upgrade"), _("Re-use your old preferences, palettes and shortcuts?")))
+        if(confirm (_("Denemo Upgrade"), _("Re-use your old preferences, commands, palettes and shortcuts?")))
             localrc =  g_build_filename (Denemo.old_user_data_dir, PREFS_FILE, NULL);
         else
-        Denemo.old_user_data_dir = NULL;
-
+			Denemo.old_user_data_dir = NULL;
     }
   /* Reasonable default values */
 
@@ -102,6 +101,7 @@ initprefs (void)
   ret->cursor_highlight = TRUE;
   ret->return_key_is_special = TRUE;
   ret->newbie = TRUE;
+  ret->noswitchback = TRUE;
   ret->persistence = TRUE;
   ret->learning = TRUE;
   ret->menunavigation = FALSE;
@@ -343,6 +343,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
         READBOOLXMLENTRY (cursor_highlight)
         READBOOLXMLENTRY (return_key_is_special)
         READBOOLXMLENTRY (newbie)
+        READBOOLXMLENTRY (noswitchback)
         READBOOLXMLENTRY (learning)
         READBOOLXMLENTRY (applytoselection)
         READBOOLXMLENTRY (quickshortcuts)
@@ -429,6 +430,7 @@ get_bool_pref (gchar * prefname)
     GETBOOLPREF (cursor_highlight)
     GETBOOLPREF (return_key_is_special)
     GETBOOLPREF (newbie)
+    GETBOOLPREF (noswitchback)
     GETBOOLPREF (learning)
     GETBOOLPREF (applytoselection)
     GETBOOLPREF (quickshortcuts)
@@ -806,6 +808,7 @@ writeXMLPrefs (DenemoPrefs * prefs)
     WRITEBOOLXMLENTRY (cursor_highlight)
     WRITEBOOLXMLENTRY (return_key_is_special)
     WRITEBOOLXMLENTRY (newbie)
+    WRITEBOOLXMLENTRY (noswitchback)
     WRITEBOOLXMLENTRY (learning)
     WRITEBOOLXMLENTRY (applytoselection)
     WRITEBOOLXMLENTRY (quickshortcuts)
