@@ -70,6 +70,8 @@ draw_rest (cairo_t * cr, gint duration, gint numdots, gint xx, gint y, DenemoGra
 
   static gunichar rest_char[SMALLESTDURATION + 1] = { 0x20, 0x21, 0x27, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2d      /*note no glyph for this one! */
   };
+  if (duration > SMALLESTDURATION)
+	duration = 0;
   if (override_rest)
     drawbitmapinverse_cr (cr, override_rest, xx + gx + restwidths[0] - override_rest->width / 2, y + restoffsets[0] + gy - override_rest->height / 2, FALSE);
   else
@@ -91,12 +93,12 @@ draw_notehead (cairo_t * cr, note * thenote, gint duration, gint numdots, gint x
      three elements.  The change has defeated what had been semi-elegance;
      grrr.  */
 
-  static gint headwidths[6] = { WHOLEHEAD_WIDTH, HALFHEAD_WIDTH,
+  static gint headwidths[7] = { WHOLEHEAD_WIDTH, HALFHEAD_WIDTH,
     NOTEHEAD_WIDTH, DIAMOND_WIDTH,
-    CROSS_WIDTH, HARMONIC_WIDTH,
+    CROSS_WIDTH, HARMONIC_WIDTH, DIAMOND_WIDTH,
   };
 
-  static gunichar head_char[6] = { 0x54, 0x55, 0x56, 0x58, 0x64, 0x92
+  static gunichar head_char[7] = { 0x54, 0x55, 0x56, 0x93, 0x64, 0x92, 0x56
   };
   gint height = thenote->y;
   gint noteheadtype = 1;
@@ -115,9 +117,9 @@ draw_notehead (cairo_t * cr, note * thenote, gint duration, gint numdots, gint x
     {
       noteheadtype = 5;
     }
-  else if (thenote->noteheadtype == DENEMO_DIAMOND_NOTEHEAD)
+  else if (thenote->noteheadtype == DENEMO_BLACK_NOTEHEAD)
     {
-      noteheadtype = 3;
+      noteheadtype = 2;
     }
   if (duration < 0)
     {
@@ -275,7 +277,7 @@ draw_chord (cairo_t * cr, objnode * curobj, gint xx, gint y, gint mwidth, gint *
         {
           note *thenote = (note *) thechord.notes->data;
           cairo_translate (cr, xx, y + thenote->y);
-          cairo_scale (cr, 0.8, 0.8);
+          cairo_scale (cr, 0.8, thechord.notes->next?1.0:0.8); //do not scale height if it is a grace note chord
           cairo_translate (cr, -xx, -(y + thenote->y));
         }
       //g_debug("Invisible is %d\n", mudelaitem->isinvisible);
