@@ -426,8 +426,8 @@ open_for_real (gchar * filename, DenemoProject * gui, DenemoSaveType template, I
                        while (!found)
                         { gchar *message = g_strdup_printf ("%s%s", _("Not Found: "), filename);
                           gchar *path = NULL; 
-                          if (choose_option (message, _("Browse for this file"), _("Ignore it")))
-							path = choose_directory (_("Give Toplevel Directory"), Denemo.prefs.denemopath->str, NULL);
+                          if (choose_option (message, _("Browse for this file's location"), _("Ignore it")))
+							path = choose_directory (_("Click Open when you have located the file"), Denemo.prefs.denemopath->str, NULL);
                           if (path)
                             {
                               found = try_to_find_file (path, filename);
@@ -483,6 +483,7 @@ open_for_real (gchar * filename, DenemoProject * gui, DenemoSaveType template, I
       if (!template)
         {                       // not a template
           update_file_selection_path (filename);
+          chdir (g_path_get_dirname (filename));
           if (type == REPLACE_SCORE)
             {
               if (xml){
@@ -731,6 +732,7 @@ filesel_save (DenemoProject * gui, const gchar * file_name, gint format_id, Dene
     {
       if(!Denemo.non_interactive)
         set_project_filename (gui, file);
+      chdir (g_path_get_dirname (file));
     }    
     
     
@@ -957,14 +959,24 @@ set_current_folder (GtkWidget * file_selection, DenemoSaveType template)
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_selection), fallback);
     }
 }
-
+//choose a file for reading/writing with title message
+//default directory is location
+//allowed extensions ext and/or exts
 gchar *
-file_dialog (gchar * message, gboolean type, gchar * location)
+file_dialog (gchar * message, gboolean read, gchar * location, gchar *ext, GList *exts)
 {
   GtkWidget *file_selection;
   gchar *filename;
-  file_selection = gtk_file_chooser_dialog_new (message, GTK_WINDOW (Denemo.window), type ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE, _("_Cancel"), GTK_RESPONSE_REJECT, type ? _("_Open") : _("_Save"), GTK_RESPONSE_ACCEPT, NULL);
-
+  file_selection = gtk_file_chooser_dialog_new (message, GTK_WINDOW (Denemo.window), read ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE, _("_Cancel"), GTK_RESPONSE_REJECT, read ? _("_Open") : _("_Save"), GTK_RESPONSE_ACCEPT, NULL);
+  GtkFileFilter *filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, _("Commands or Shortcuts"));
+  if (ext)
+	gtk_file_filter_add_pattern (filter, ext);
+  GList *g;
+  for (g=exts;g;g=g->next)
+	gtk_file_filter_add_pattern (filter, (gchar*)g->data);
+  if (ext || exts) 
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_selection), filter);
   if (location)
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_selection), location);
   gtk_dialog_set_default_response (GTK_DIALOG (file_selection), GTK_RESPONSE_ACCEPT);
@@ -974,6 +986,13 @@ file_dialog (gchar * message, gboolean type, gchar * location)
   else
     filename = NULL;
   gtk_widget_destroy (file_selection);
+  if ((!read) && g_file_test (filename, G_FILE_TEST_EXISTS))
+	{
+	 gchar *primary = g_strdup_printf (_("A file with the name %s already exists"), filename);
+	 if (!confirm (primary, _("Do you want to replace it?")))
+		filename = NULL;
+	 g_free (primary);	
+	}
   return filename;
 }
 
@@ -1268,12 +1287,21 @@ file_dialog_response (GtkWidget * dialog, gint response_id, struct FileDialogDat
   /*set default folder for saving */\
   set_current_folder(file_selection, template==SAVE_TEMPLATE?SAVE_TEMPLATE:SAVE_NORMAL);\
   \
-  /* assign title */ \
-  gchar *title = get_scoretitle();\
-  if (title)\
-    { \
-      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_selection), title);\
-    } \
+  /* Pre-fill filename if current file has a name */\
+  if (Denemo.project->filename && Denemo.project->filename->len > 0) {\
+    gchar *basename = g_path_get_basename (Denemo.project->filename->str);\
+    /* Strip existing extension and add the appropriate one for the format */\
+    gchar *basename_no_ext = g_strdup (basename);\
+    gchar *dot = strrchr (basename_no_ext, '.');\
+    if (dot != NULL && dot != basename_no_ext) {\
+      *dot = '\0';\
+    }\
+    gchar *suggested_name = g_strconcat (basename_no_ext, FORMAT_EXTENSION(format_id), NULL);\
+    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_selection), suggested_name);\
+    g_free (basename);\
+    g_free (basename_no_ext);\
+    g_free (suggested_name);\
+  }\
   \
   filter = gtk_file_filter_new ();\
   gtk_file_filter_set_name (filter, FORMAT_DESCRIPTION(format_id));\
