@@ -31,6 +31,9 @@
 #include "export/exportlilypond.h"
 #include "core/utils.h"
 
+#define LATER_VERSION_LILYPOND_INCLUDE_DIR "actions/lilypond/2.20"
+
+
 gint LilyPond_stderr;       //A file descriptor to pipe for LilyPond's stderr
 GError *lily_err;
 GPid previewerpid;
@@ -101,12 +104,9 @@ void initialize_print_status (void)
 static void
 advance_printname ()
 {
-
-
   Denemo.printstatus->cycle = !Denemo.printstatus->cycle;
   /*gint success =*/ g_unlink (Denemo.printstatus->printname_pdf[Denemo.printstatus->cycle]);
   g_unlink (Denemo.printstatus->printname_svg[Denemo.printstatus->cycle]);
-
   //g_debug("Removed old pdf file %s %d\n",Denemo.printstatus->printname_pdf[Denemo.printstatus->cycle], success);
 }
 
@@ -237,9 +237,15 @@ gchar *get_lilypond_include_dir (void)
 	if (Denemo.lilypond_installed_version)
 	{
 	  lilyversion installed_version = string_to_lilyversion (Denemo.lilypond_installed_version);
-	  lilyversion check_version = string_to_lilyversion ("2.18.0");
+	  lilyversion check_version;
+	  
+	  check_version = string_to_lilyversion ("2.22.0");
 	  if (version_check (check_version, installed_version) == GREATER)
-		return LATER_VERSION_LILYPOND_INCLUDE_DIR;
+		return "actions/lilypond/2.24";	  //no upgrades were needed for 2.22 so we use 2.20 for that
+		
+	  check_version = string_to_lilyversion ("2.18.0");
+	  if (version_check (check_version, installed_version) == GREATER)
+		return "actions/lilypond/2.20";
 	}
   return LILYPOND_INCLUDE_DIR;
 }
@@ -325,6 +331,11 @@ static gchar * get_error_point (gchar *bytes, gint *line, gint *col)
 {
     gchar *epoint;
     gchar *message = bytes;
+    gchar *error = g_strrstr (bytes, "ERROR:");
+    if (error) {
+		return error;
+	}
+    
     while (*message)
         {
             while (*message && (*message!=':')) message++;
@@ -375,6 +386,14 @@ static gchar * get_error_point (gchar *bytes, gint *line, gint *col)
                         continue;
                      }
         }
+	error = g_strrstr (bytes, "programming error:");
+    if (error) {
+		return error;
+	}
+	error = g_strrstr (bytes, "fatal error:");
+    if (error) {
+		return error;
+	}
     return NULL;
 }
 void
@@ -387,11 +406,20 @@ process_lilypond_errors (gchar * filename)
   gchar *bytes;
   gint numbytes = g_file_get_contents (logfile, &bytes, NULL, NULL);
   g_free (logfile);
+  //g_print("\nLilyPond error messages\n8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8>< %s \n8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><\n", bytes);
   if (bytes)
    numbytes=strlen (bytes);
   else
     return;
-  //g_print("\nLilyPond error messages\n8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8>< %s \n8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><8><\n", bytes);
+  gchar *page_system_info = g_strrstr (bytes, "DenemoInfo=");
+  if (page_system_info)
+	{
+		sscanf (page_system_info, "DenemoInfo=%d,%d", &Denemo.printstatus->pages, &Denemo.printstatus->systems);
+	}
+  else
+	Denemo.printstatus->pages = 0;
+  
+  
   gint line, column;
   epoint=get_error_point (bytes, &line, &column);
   if (epoint)
@@ -408,7 +436,11 @@ process_lilypond_errors (gchar * filename)
           set_lily_error (line + 1, column);
         }
       goto_lilypond_position (line + 1, column);
-      Denemo.printstatus->invalid = 2;      //print_is_valid = FALSE;
+      
+      if (g_strrstr (epoint,"error:"))
+		Denemo.printstatus->invalid = 2;
+      else
+		Denemo.printstatus->invalid = 4;
       if (Denemo.printarea)
         gtk_widget_queue_draw (Denemo.printarea);
       // FIXME this causes a lock-up     warningdialog(_("Typesetter detected errors. Cursor is position on the error point.\nIf in doubt delete and re-enter the measure."));
@@ -441,7 +473,7 @@ open_viewer (gint status, gchar * filename)
   gchar *printfile;
   gchar **arguments;
   progressbar_stop ();
-  console_output (_("Done"));
+  console_output (_("Done\n"));
   g_spawn_close_pid (Denemo.printstatus->printpid);
   Denemo.printstatus->printpid = GPID_NONE;
   //normal_cursor();
@@ -524,6 +556,7 @@ static gint
 run_lilypond (gchar ** arguments)
 {
   static gboolean old_error = FALSE;
+  static gchar* dummystderr; gchar* dummystdout;
   gint error = 0;
   if (old_error)
     {
@@ -532,14 +565,24 @@ run_lilypond (gchar ** arguments)
       write_input_status ();
       old_error = FALSE;  
     }
+  Denemo.printstatus->pages = 0;
+  // Don't show progress bar if Print View is visible - the user is watching it already
+  // and the progress bar steals focus, pushing Print View behind main window
+  gboolean printview_visible = (Denemo.printarea && gtk_widget_get_visible(gtk_widget_get_toplevel(Denemo.printarea)));
   if (Denemo.printstatus->background == STATE_NONE)
-    progressbar (_("Denemo Typesetting"), call_stop_lilypond);
+    {
+      if (printview_visible)
+        start_typeset_progress ();  // Show spinner in Print View toolbar
+      else
+        progressbar (_("Denemo Typesetting"), call_stop_lilypond);  // Show separate progress window
+    }
   if (lily_err)
     {
       g_warning ("Old error message from launching lilypond still present - message was %s\nDiscarding...", lily_err->message);
       g_error_free (lily_err);
       lily_err = NULL;
     }
+
   console_output (NULL);
   console_output (_("Typesetting ..."));
    gboolean lilypond_launch_success;
@@ -550,8 +593,8 @@ if (Denemo.non_interactive)
                  G_SPAWN_SEARCH_PATH,
                  NULL,    /* child setup func */
                  NULL,    /* user data */
-                 NULL,    /* stdout */
-                 NULL, /* stderr */
+                 &dummystdout,    /* stdout */
+                 &dummystderr, /* stderr */
                  &lilypond_launch_success,
                  &lily_err);
 else
@@ -563,8 +606,8 @@ else
                                                                NULL,    /* user data */
                                                                &Denemo.printstatus->printpid,
                                                                NULL,
-                                                               NULL,    /* stdout */
-                                                               NULL, /* stderr */
+                                                               (gint*)&dummystdout,    /* stdout */
+                                                               (gint*)&dummystderr, /* stderr */
                                                                &lily_err);
 
 
@@ -618,9 +661,10 @@ run_lilypond_for_pdf (gchar * filename, gchar * lilyfile)
 {
  // if(!include) initialize_lilypond_includes();
   /*arguments to pass to lilypond to create a pdf for printing */
+  gchar *logfile = g_strdup_printf ("-dlog-file=%s", filename);
   gchar *arguments[] = {
     Denemo.prefs.lilypath->str,
-    "-dgui",
+    logfile,
     "--loglevel=WARN",
     "--pdf",
     local_include,
@@ -631,6 +675,7 @@ run_lilypond_for_pdf (gchar * filename, gchar * lilyfile)
     NULL
   };
   run_lilypond (arguments);
+  g_free (logfile);
 }
 //synchronous generation of outfile.pdf from input lilyfile
 void generate_pdf_from_lily_file (gchar *lilyfile, gchar *outfile)
@@ -644,12 +689,11 @@ void generate_pdf_from_lily_file (gchar *lilyfile, gchar *outfile)
 static void
 run_lilypond_for_svg (gchar * filename, gchar * lilyfile)
 {
-  //  if(!include) initialize_lilypond_includes();
-
+  gchar *logfile = g_strdup_printf ("-dlog-file=%s", filename);
   /*arguments to pass to lilypond to create a svg for printing */
   gchar *arguments[] = {
     Denemo.prefs.lilypath->str,
-    "-dgui",
+     logfile,
     "--loglevel=WARN",
      "-dno-point-and-click", "-ddelete-intermediate-files", "-dbackend=svg",
     local_include,
@@ -660,6 +704,7 @@ run_lilypond_for_svg (gchar * filename, gchar * lilyfile)
     NULL
   };
   run_lilypond (arguments);
+  g_free (logfile);
 }
 /*  create pdf of current score, optionally restricted to voices/staffs whose name match the current one.
  *  generate the lilypond text (on disk)
