@@ -20,7 +20,7 @@ gboolean print_typeset_pdf (void);
 void typeset_current_layout (void);
 void typeset_current_movement (void);
 void implement_show_print_view (gboolean refresh_if_needed);
-void install_printpreview (GtkWidget * vbox);
+void install_printpreview (void);
 void refresh_print_view (gboolean interactive);
 void printview_finished (G_GNUC_UNUSED GPid pid, G_GNUC_UNUSED gint status, gboolean print);
 void print_from_print_view (gboolean all_movements);
@@ -29,5 +29,8 @@ void unpause_continuous_typesetting (void);
 void pause_continuous_typesetting (void);
 void display_printhistory (void);
 void delete_conditions (DenemoProject *gui);
-void  set_condition (DenemoOmissionCriterion *condition);
+void  set_condition (DenemoInclusionCriterion *condition);
+void create_new_inclusion_criterion (gchar *name);
+void start_typeset_progress (void);
+void stop_typeset_progress (void);
 #endif
