@@ -35,9 +35,11 @@
 
 
 #ifndef USE_EVINCE
+#ifndef USE_ATRIL
 void set_continuous_typesetting (gboolean setting){return FALSE;}
 gboolean continuous_typesetting () {return FALSE;}
 #endif
+#endif
 
 static gint changecount = -1;   //changecount when the playback typeset was last created
 static gboolean RightButtonPressed = FALSE;
@@ -51,6 +53,8 @@ static gboolean PartOnly = FALSE;
 static GtkAdjustment *VAdj = NULL;
 static gdouble ScrollTime = -1.0;
 static GtkWidget *ClearScrollPointsButton = NULL;
+static gchar *LilyPondSVGPageSuffix = "-"; // earlier LilyPond versions output -page-n.svg for nth page of SVG
+static GtkWidget *playalong_toggle;
 typedef struct Timing {
     gdouble time;
     gdouble duration;
@@ -130,10 +134,20 @@ static void
 show_playback_view (void)
 {
     GtkWidget *w = gtk_widget_get_toplevel (Denemo.playbackview);
+	if (Denemo.project->midi_destination & MIDIPLAYALONG)
+		{
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (playalong_toggle), TRUE);
+			Denemo.project->midi_destination ^= MIDIPLAYALONG;
+		}
+	else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (playalong_toggle), FALSE);
+
     if (!gtk_widget_get_visible (w))
         set_toggle (TogglePlaybackView_STRING, TRUE);
     else
-        gtk_window_present (GTK_WINDOW (w));
+		gtk_window_present (GTK_WINDOW (w));
+		
+	gtk_widget_hide (get_playalong_button());//only show toggle in playback view if it is made visible.
 }
 
 
@@ -203,6 +217,14 @@ DenemoObject *get_object_for_time (gdouble time, gboolean start)
         }
     return NULL;
 }
+
+static 
+gint playalong_highlight = 0xb5;
+void highlight_playback_note (gint color)
+{
+	playalong_highlight = color;//g_print ("set color");
+	gtk_widget_queue_draw (Denemo.playbackview);
+}
 //over-draw the evince widget with padding etc ...
 static gboolean
 overdraw_print (cairo_t * cr)
@@ -277,6 +299,7 @@ overdraw_print (cairo_t * cr)
                        continue;
            if (time > (this - 0.1))
                     { // g_print ("draw note at %.2f %.2f\n", ((Timing *)((g)->data))->x  - (PRINTMARKER/5)/4, ((Timing *)((g)->data))->y - (PRINTMARKER/5)/2 );
+						cairo_set_source_rgba (cr, playalong_highlight/255.0, 0x0/255.0, playalong_highlight/255.0, 0.5);
                         cairo_rectangle (cr, ((Timing *)((g)->data))->x  - (PRINTMARKER/5)/4, ((Timing *)((g)->data))->y - (PRINTMARKER/5)/2, PRINTMARKER/5, PRINTMARKER/5);
                         if(!drew_rectangle)
                             LastTiming = g;
@@ -412,8 +435,8 @@ static void compute_timings (gchar *base, GList *ids)
                     {//breve duration is log = -1  if longa log = -2
 						//e.g.  0	rest	log = -1	2000	point-and-click 9 21
 					gchar basedur[10];
-					if (1 == fscanf (fp, "%4s", basedur))
-						{
+					if (1 == fscanf (fp, "%9s", basedur))
+						{ //g_print("basedur is %s\n", basedur);
 						if (!strcmp (basedur, "log"))
 							fscanf (fp, " %*s %*s");
                         if (!strcmp (type, "note"))
@@ -494,6 +517,9 @@ static void compute_timings (gchar *base, GList *ids)
             } //if events file
     else
         {
+			gchar *msg = g_strdup_printf ("Failure! No events file created by LilyPond - file too big???.");
+			warningdialog (msg);
+			g_free (msg);
           g_critical ("Unable to open file %s Playback View will not work", events);  
         }
 
@@ -583,7 +609,7 @@ static gint get_number_of_pages (gchar *base)
     gint i;
     for (i=1;i<10;i++)
         {
-            gchar *filename = g_strdup_printf ("%s%s%d%s", Denemo.printstatus->printbasename[Denemo.printstatus->cycle], "-page-", i, ".svg");
+            gchar *filename = g_strdup_printf ("%s%s%d%s", Denemo.printstatus->printbasename[Denemo.printstatus->cycle], "-", i, ".svg");
             if (!(g_file_test (filename, G_FILE_TEST_EXISTS)))
                 {
                     g_free(filename);
@@ -594,6 +620,19 @@ static gint get_number_of_pages (gchar *base)
         }
    return i-1;
 }
+
+static gchar *get_script (gboolean part)
+{
+    gint w, h;
+    GtkWidget *win = gtk_widget_get_toplevel (Denemo.playbackview); 
+    gtk_window_get_size (GTK_WINDOW(win), &w, &h); //why does get_window_size() not work???
+    //g_print ("Width = %d\n", w);//default 709
+    w = 20*w/709.0;
+    return g_strdup_printf ("(d-PlaybackView (list %s \"%d\" \"100\"))", part?"#t":"#f", w);	
+}
+
+
+
 static gboolean
 set_playback_view (void)
 {
@@ -609,11 +648,21 @@ set_playback_view (void)
           g_free (filename);
           if (num_pages>0) //recursion failed, give up
             {
+				gchar *msg = g_strdup_printf ("Failure! LilyPond - stopped after %d\n pages.", num_pages);
+				warningdialog (msg);
+				g_free (msg);
                 num_pages = 0;
                 g_warning ("Unable to get the right page length\n");
                 return FALSE;
             }
-          filename = g_strconcat (Denemo.printstatus->printbasename[Denemo.printstatus->cycle], "-page-1.svg", NULL);
+          filename = g_strconcat (Denemo.printstatus->printbasename[Denemo.printstatus->cycle], "-1.svg", NULL);
+          if (g_file_test (filename, G_FILE_TEST_EXISTS))
+			LilyPondSVGPageSuffix = "-";
+		  else
+			LilyPondSVGPageSuffix = "-page-";
+		  g_free (filename);
+		  filename = g_strconcat (Denemo.printstatus->printbasename[Denemo.printstatus->cycle], LilyPondSVGPageSuffix, "1.svg", NULL);
+			
           if (g_file_test (filename, G_FILE_TEST_EXISTS))
                 {
                     g_free (filename);
@@ -623,10 +672,9 @@ set_playback_view (void)
                         g_warning ("Unable to determine number of pages\n");
                         return FALSE;
                         }
-                    gchar *scheme = g_strdup_printf ("%s%s%s%d%s", "(d-PlaybackView \"(list ", PartOnly?"#t":"#f", " \\\"20\\\" \\\"" , 100 * num_pages, "\\\")\")");
-                    //g_print ("Scheme created: %s for %d pages\n", scheme, num_pages);
-                    call_out_to_guile (scheme);
-                    g_free (scheme);
+					gchar *command = get_script (PartOnly);
+					call_out_to_guile (command);//this installs the temporary directives to typeset svg and thenthen calls LilyPond
+					g_free (command);
                     return FALSE;
                 }
          g_free (filename);
@@ -695,13 +743,22 @@ static void clear_scroll_points (void)
 
 static void help_scroll_points (void)
 {
-    infodialog (_("This the Playback View Window. Click on a note to play from that note to the end. Click again to stop play. Drag between two notes to play from the first to the last, shift drag to create a loop.\nShift-Click on a note to position the Denemo cursor on that note in the Denemo Display.\n For simple scrolling check the box. For more sophisticated control right click on a note when you have scrolled the page to the position you want it to be at when it is playing.\nFirst right click at the start of the second system (this means that the music will not scroll before that); then scroll to position the end and right click the first note of the last system of the piece.\nTo delete a scroll point right-click on it.\nIf there are changes of pace then set extra scroll points to control the scrolling in more detail."));
+    infodialog (_("This is the Playback View Window. Click on a note to play from that note to the end. \
+Click again to stop play. Drag between two notes to play from the first to the last, shift drag to create a loop. \
+Shift-Click on a note to position the Denemo cursor on that note in the Denemo Display. \
+For simple scrolling check the box. For more sophisticated control right click on a note when you have scrolled the page to the position you want it to be at when it is playing. \
+First right click at the start of the second system (this means that the music will not scroll before that); then scroll to position the end and right click the first note of the last system of the piece. \
+To delete a scroll point right-click on it.\nIf there are changes of pace then set extra scroll points to control the scrolling in more detail. \
+For PlayAlong playback left click on the note you want to start playing at - it will sound immediately, but you must still play it on your MIDI keyboard for the playback to advance. \
+If you want an intro use Ctrl-left click - it will play from the bar before until it plays the note you clicked, \
+you have to play that note and then the following ones to continue the playback. \
+You can shape the part you are playing by playing staccato or using rubato to slow up the playback."));
 }
 static void
 playbackview_finished (G_GNUC_UNUSED GPid pid, G_GNUC_UNUSED gint status, gboolean print)
 {
   progressbar_stop ();
-
+  console_output (_("Done (Playback View)"));
   g_spawn_close_pid (Denemo.printstatus->printpid);
   //g_print ("background %d\n", Denemo.printstatus->background);
   if (Denemo.printstatus->background == STATE_NONE)
@@ -721,6 +778,7 @@ playbackview_finished (G_GNUC_UNUSED GPid pid, G_GNUC_UNUSED gint status, gboole
       gdouble total_time;
       changecount = Denemo.project->movement->changecount;
       total_time = load_lilypond_midi (NULL, AllPartsTypeset);//g_print ("MIDI file total time = %.2f\n", total_time);
+ 
       Denemo.project->movement->smfsync = Denemo.project->movement->changecount;
       AllPartsTypeset = FALSE;
   }
@@ -813,7 +871,7 @@ void delete_svgs (void) {
     gint i;
     for (i=1;i<10;i++)
         {
-            gchar *filename = g_strdup_printf ("%s%s%d%s", Denemo.printstatus->printbasename[cycle], "-page-", i, ".svg");
+            gchar *filename = g_strdup_printf ("%s%s%d%s", Denemo.printstatus->printbasename[cycle], LilyPondSVGPageSuffix, i, ".svg");
             if (!g_file_test (filename, G_FILE_TEST_EXISTS))
                 {
                  //g_print ("No file %s\n", filename);
@@ -1121,9 +1179,14 @@ static void button_release (GtkWidget *event_box, GdkEventButton *event)
                 {
 
                     if ((timing->col == Locationx) && (timing->line == Locationy))
-                        {
+                        {//g_print ("Playing from cursor to end or previous bar to cursor for playalong\n");
                             if (!shift_held_down())
-                              call_out_to_guile ("(d-DenemoPlayCursorToEnd)");
+								{
+										if (control_held_down())
+											call_out_to_guile ("(DenemoPlayIntroToEnd)");
+										else
+											call_out_to_guile ("(d-DenemoPlayCursorToEnd)");
+								}
                             else
                                 call_out_to_guile ("(d-PlayMidiNote 67 255 9 100)");
                             //g_print ("Found same line %d column %d\n", timing->line, timing->col);
@@ -1134,7 +1197,7 @@ static void button_release (GtkWidget *event_box, GdkEventButton *event)
                                 {
                                     gboolean found = goto_lilypond_position (timing->line, timing->col);//g_print ("y %d Lefty %d\n", y, LeftButtonY);
 
-                                    call_out_to_guile ("(if (not (d-NextChord)) (d-MoveCursorRight))(DenemoSetPlaybackEnd)");
+                                    call_out_to_guile ("(if (not (d-NextChord)) (d-MoveCursorRight)(d-MoveCursorLeft))(DenemoSetPlaybackEnd)");
                                         //g_print ("Set playback end to %d column %d\n", timing->line, timing->col);
                                     Denemo.project->movement->smfsync = Denemo.project->movement->changecount;
                                     if (shift_held_down())
@@ -1174,18 +1237,24 @@ static void play_button (void)
     Denemo.project->movement->smfsync = Denemo.project->movement->changecount;
     call_out_to_guile ("(d-Performance)");
 }
+
+
+
 static void part_button (void)
 {
     PartOnly = TRUE;
     if (Denemo.project->movement->smf)
         AllPartsTypeset = confirm ( _("MIDI Already Present"), _("Keep this music while typesetting current part?"));
-    call_out_to_guile ("(d-PlaybackView 'part)");//this installs the temporary directives to typeset svg and then
-
+	gchar *command = get_script (TRUE);
+    call_out_to_guile (command);//this installs the temporary directives to typeset svg and thenthen calls LilyPond
+	g_free (command);
 }
 static void movement_button (void)
 {
     PartOnly = FALSE;
-    call_out_to_guile ("(d-PlaybackView #f)");//this installs the temporary directives to typeset svg and then
+    gchar *command = get_script (PartOnly);
+    call_out_to_guile (command);//this installs the temporary directives to typeset svg and then calls LilyPond
+    g_free (command);
 }
 
 
@@ -1254,9 +1323,15 @@ install_svgview (GtkWidget * top_vbox)
     gtk_widget_set_tooltip_markup (button, _( "Sets/Unsets automatic scrolling. The scrolling can still be manually adjusted if it is too fast/slow."));
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-  g_signal_connect_swapped (G_OBJECT (button), "toggled", G_CALLBACK (scroll_toggle), NULL);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    g_signal_connect_swapped (G_OBJECT (button), "toggled", G_CALLBACK (scroll_toggle), NULL);
 
+    playalong_toggle = (GtkWidget*)gtk_check_button_new_with_label (_("Play Along"));
+    gtk_widget_set_tooltip_markup (playalong_toggle, _( "Sets/Unsets Play Along playback."));
+	if (Denemo.project && Denemo.project->midi_destination & MIDIPLAYALONG)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (playalong_toggle), TRUE);
+	gtk_box_pack_start (GTK_BOX (hbox), playalong_toggle, FALSE, FALSE, 0);
+    g_signal_connect_swapped (G_OBJECT (playalong_toggle), "toggled", G_CALLBACK (pb_playalong), get_playalong_button());
 
 
 
@@ -1284,7 +1359,23 @@ install_svgview (GtkWidget * top_vbox)
   gtk_container_add (GTK_CONTAINER (top_vbox), main_vbox);
 
   GtkWidget *score_and_scroll_win = gtk_scrolled_window_new (NULL, NULL);
-  Denemo.playbackview = (GtkWidget *) gtk_image_new ();
+ // Denemo.playbackview = (GtkWidget *) gtk_image_new ();
+  
+  gchar *filename = g_build_filename (get_system_data_dir (), COMMANDS_DIR, "graphics", "PlaybackView.svg", NULL);
+  
+#ifdef G_OS_WIN32
+    GError *err = NULL;
+    err = NULL;
+    GdkPixbuf *pb = rsvg_pixbuf_from_file (filename, &err);
+    if(pb)
+         Denemo.playbackview = gtk_image_new_from_pixbuf (pb);
+     else
+        g_warning ("\n\nThe rsvg pixbuf load of %s gave error: %s\n\n", filename, err?err->message: "no error return");
+#else
+        Denemo.playbackview = gtk_image_new_from_file (filename);
+#endif 
+    g_free (filename);
+  
     VAdj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(score_and_scroll_win));
     // gtk_container_add (GTK_CONTAINER (score_and_scroll_win), Denemo.playbackview);
     //instead use an hbox to prevent the GtkImage widget expanding beyond the image size, which then causes positioning errors.
